using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;
using System.ComponentModel;

namespace CustomControls.TemplatedControls
{
	[ParseChildren(true)]
	public class TemplatedControl6 : WebControl, INamingContainer
	{
		#region  
		private static readonly object EventSelectedIndexChanged = new object();
		private static readonly object EventItemCreated = new object();
		private static readonly object EventItemDataBound = new object();
		private static readonly object EventItemCommand = new object();
		#endregion

		private ITemplate itemTemplate = null;
		private ITemplate alternatingTemplate = null;
		private IEnumerable dataSource = null;

		#region Ӽ
		public IEnumerable DataSource {
			get {return dataSource;}
			set {dataSource = value;}
		}

		[PersistenceMode(PersistenceMode.InnerProperty),
		TemplateContainer(typeof(TemplateItem))]
		public ITemplate ItemTemplate {
			get {return itemTemplate;}
			set {itemTemplate = value;}
		}

		[PersistenceMode(PersistenceMode.InnerProperty),
		TemplateContainer(typeof(TemplateItem))]
		public ITemplate AlternatingTemplate {
			get {return alternatingTemplate;}
			set {alternatingTemplate = value;}
		}

		public virtual int SelectedIndex {
			get {
				object o = ViewState["SelectedIndex"];
				return (o == null) ? -1 : (int)o;
			}
			set {
				if (value < -1) 
					throw new ArgumentOutOfRangeException();

				ViewState["SelectedIndex"] = value;
			}
		}

		private int ItemCount {
			get {
				object o = ViewState["itemCount"];
				return (o == null) ? 0 : (int)o;
			}
			set {ViewState["itemCount"] = value;}
		}
		#endregion

		#region ̺Ʈ
		protected override bool OnBubbleEvent(object source, EventArgs e) 
		{
			if (e is TemplatedControlCommandEventArgs) {
				TemplatedControlCommandEventArgs ce = (TemplatedControlCommandEventArgs)e;

				OnItemCommand(ce);

				if (String.Compare(ce.CommandName, "Select", true) == 0) {
					SelectedIndex = ce.Item.ItemIndex;
					OnSelectedIndexChanged(EventArgs.Empty);
				}

				return true;
			}

			return false;
		}

		protected virtual void OnItemCommand(TemplatedControlCommandEventArgs e) 
		{
			TemplatedControlCommandEventHandler onItemCommandHandler = (TemplatedControlCommandEventHandler)Events[EventItemCommand];
			if (onItemCommandHandler != null) onItemCommandHandler(this, e);
		}

		protected virtual void OnItemCreated(TemplateItemEventArgs e) 
		{
			TemplateItemEventHandler onItemCreatedHandler = (TemplateItemEventHandler)Events[EventItemCreated];
			if (onItemCreatedHandler != null) onItemCreatedHandler(this, e);
		}

		protected virtual void OnItemDataBound(TemplateItemEventArgs e) 
		{
			TemplateItemEventHandler onItemDataBoundHandler = (TemplateItemEventHandler)Events[EventItemDataBound];
			if (onItemDataBoundHandler != null) onItemDataBoundHandler(this, e);
		}

		protected virtual void OnSelectedIndexChanged(EventArgs e) 
		{
			EventHandler handler = (EventHandler)Events[EventSelectedIndexChanged];
			if (handler != null) handler(this, e);
		}

		public event TemplatedControlCommandEventHandler ItemCommand {
			add {Events.AddHandler(EventItemCommand, value);}
			remove {Events.RemoveHandler(EventItemCommand, value);}
		}

		public event TemplateItemEventHandler ItemCreated {
			add {Events.AddHandler(EventItemCreated, value);}
			remove {Events.RemoveHandler(EventItemCreated, value);}
		}

		public event TemplateItemEventHandler ItemDataBound {
			add {Events.AddHandler(EventItemDataBound, value);}
			remove {Events.RemoveHandler(EventItemDataBound, value);}
		}

		public event EventHandler SelectedIndexChanged {
			add {Events.AddHandler(EventSelectedIndexChanged, value);}
			remove {Events.RemoveHandler(EventSelectedIndexChanged, value);}
		}
#endregion

		protected override void CreateChildControls()
		{
			Controls.Clear();

			if(ItemCount != 0)
				CreateControlHierarchy(false);
		}

		public override void DataBind()
		{
			//   ε ϱ   ű
			base.DataBind();

			Controls.Clear();

			if (dataSource != null) {
				if(HasChildViewState)
					ClearChildViewState();

				CreateControlHierarchy(true);
				ChildControlsCreated = true;
			}
		}

		private void CreateControlHierarchy(bool useDataSource) 
		{
			IEnumerable datSrc = null;

			if (useDataSource) 
				datSrc = this.dataSource;
			else 
				datSrc = new DummyDataSource(ItemCount);

			if (datSrc != null) {
				int count = 0;
				foreach(object dataItem in datSrc) {
					ListItemType itemType;
					ITemplate template;
					
					if (count % 2 == 0 || alternatingTemplate == null) {
						itemType = ListItemType.Item;
						template = itemTemplate;
					}
					else {
						itemType = ListItemType.AlternatingItem;
						template = alternatingTemplate;
					}

					TemplateItem item = new TemplateItem(count, itemType, dataItem);
					TemplateItemEventArgs e = new TemplateItemEventArgs(item);

					OnItemCreated(e);
					template.InstantiateIn(item);

					if (useDataSource) {
						item.DataBind();
						OnItemDataBound(e);
					}
					Controls.Add(item);
					count++;
				}

				if (useDataSource) 
					ItemCount = count;
			}
		}

		internal class DummyDataSource : IEnumerable
		{
			private int itemCount;

			public DummyDataSource(int itemCount) {
				this.itemCount = itemCount;
			}

			public IEnumerator GetEnumerator() {
				return new DummyDataSourceEnumerator(itemCount);
			}

			private class DummyDataSourceEnumerator : IEnumerator 
			{
				private int count;
				private int index;

				public DummyDataSourceEnumerator(int count) {
					this.count = count;
					this.index = -1;
				}

				public object Current {
					get {return null;}
				}

				public bool MoveNext() {
					index++;
					return index < count;
				}

				public void Reset() {
					this.index = -1;
				}
			}
		}
	}

	public class TemplateItem : Control, INamingContainer 
	{
		private int itemIndex;
		private ListItemType itemType;
		private object dataItem;

		public TemplateItem(int itemIndex, ListItemType itemType, object dataItem) 
		{
			this.itemIndex = itemIndex;
			this.itemType = itemType;
			this.dataItem = dataItem;
		}

		public virtual object DataItem {
			get {return dataItem;}
			set {dataItem = value;}
		}

		public virtual int ItemIndex {
			get {return itemIndex;}
		}

		public virtual ListItemType ItemType {
			get {return itemType;}
		}

		protected override bool OnBubbleEvent(object source, EventArgs e) 
		{
			if (e is CommandEventArgs) {
				TemplatedControlCommandEventArgs args =
					new TemplatedControlCommandEventArgs(this, source, (CommandEventArgs)e);

				RaiseBubbleEvent(this, args);
				// ̺Ʈ ֻ Ʈѿ óϵ Ѵ.
				return true;
			}

			return false;
		}
	}

	//̺Ʈ    Ʈѷ Ѱ .
	public sealed class TemplatedControlCommandEventArgs : CommandEventArgs 
	{
		private TemplateItem item;
		private object commandSource;

		public TemplatedControlCommandEventArgs (TemplateItem item, object commandSource, CommandEventArgs originalArgs) :
			base(originalArgs) 
		{
			this.item = item;
			this.commandSource = commandSource;
		}

		public TemplateItem Item {
			get {return item;}
		}

		public object CommandSource {
			get {return commandSource;}
		}
	}

	public delegate void TemplatedControlCommandEventHandler(object source, TemplatedControlCommandEventArgs  e);

	//OnItemCreate OnItemDataBound ޼ҵ ڷ .
	public sealed class TemplateItemEventArgs : EventArgs 
	{
		private TemplateItem item;

		public TemplateItemEventArgs(TemplateItem item) {
			this.item = item;
		}

		public TemplateItem Item {
			get {return item;}
		}
	}

	public delegate void TemplateItemEventHandler(object sender, TemplateItemEventArgs e);
}
