using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Data;
using System.Diagnostics;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;

namespace CustomControls.TemplatedControls
{
	public class TemplatedListDesigner : TemplatedControlDesigner, IDataSourceProvider 
	{

		private DataTable dummyDataTable;
		private DataTable designTimeDataTable;
		private TemplateEditingVerb[] templateVerbs;
		private bool templateVerbsDirty;

		public TemplatedListDesigner() 
		{
			templateVerbsDirty = true;
		}

		public override bool AllowResize 
		{
			get 
			{
				// When templates are not defined, render a read-only fixed-
				// size block. Once templates are defined or are being edited, the control should allow
				// resizing.
				return TemplatesExist || InTemplateMode;
			}
		}

		public string DataSource 
		{
			get 
			{
				DataBinding binding = DataBindings["DataSource"];

				if (binding != null) 
				{
					return binding.Expression;
				}
				return String.Empty;
			}
			set 
			{
				if ((value == null) || (value.Length == 0)) 
				{
					DataBindings.Remove("DataSource");
				}
				else 
				{
					DataBinding binding = DataBindings["DataSource"];

					if (binding == null) 
					{
						binding = new DataBinding("DataSource", typeof(IEnumerable), value);
					}
					else 
					{
						binding.Expression = value;
					}
					DataBindings.Add(binding);
				}

				OnDataSourceChanged();
				OnBindingsCollectionChanged("DataSource");
			}
		}

		public override bool DesignTimeHtmlRequiresLoadComplete 
		{
			get 
			{
				// If there is a data source, look it up in the container
				// and require the document to be loaded completely.
				return (DataSource.Length != 0);
			}
		}

		protected bool TemplatesExist 
		{
			get 
			{
				return (((TemplatedList)Component).ItemTemplate != null);
			}
		}

		protected override ITemplateEditingFrame CreateTemplateEditingFrame(TemplateEditingVerb verb) 
		{
			ITemplateEditingService teService = (ITemplateEditingService)GetService(typeof(ITemplateEditingService));
			Debug.Assert(teService != null, "How did we get this far without an ITemplateEditingService");
			Debug.Assert(verb.Index == 0);

			string[] templateNames = new string[] { "ItemTemplate" };
			Style[] templateStyles = new Style[] { ((TemplatedList)Component).ItemStyle };

			ITemplateEditingFrame editingFrame =
				teService.CreateFrame(this, verb.Text, templateNames, ((TemplatedList)Component).ControlStyle, templateStyles);
			return editingFrame;
		}

		protected override void Dispose(bool disposing) 
		{
			if (disposing) 
			{
				DisposeTemplateVerbs();
			}
			base.Dispose(disposing);
		}

		private void DisposeTemplateVerbs() 
		{
			if (templateVerbs != null) 
			{
				templateVerbs[0].Dispose();

				templateVerbs = null;
				templateVerbsDirty = true;
			}
		}

		protected override TemplateEditingVerb[] GetCachedTemplateEditingVerbs() 
		{
			if (templateVerbsDirty == true) 
			{
				DisposeTemplateVerbs();

				templateVerbs = new TemplateEditingVerb[1];
				templateVerbs[0] = new TemplateEditingVerb("Item Template", 0, this);

				templateVerbsDirty = false;
			}

			return templateVerbs;
		}

		protected IEnumerable GetDesignTimeDataSource(int minimumRows, out bool dummyDataSource) 
		{
			dummyDataSource = false;

			IEnumerable selectedDataSource = ((IDataSourceProvider)this).GetResolvedSelectedDataSource();
			DataTable dataTable = designTimeDataTable;

			// Use the data table corresponding to the selected data source 
			// if possible.
			if (dataTable == null) 
			{
				if (selectedDataSource != null) 
				{
					designTimeDataTable = DesignTimeData.CreateSampleDataTable(selectedDataSource);
					dataTable = designTimeDataTable;
				}

				if (dataTable == null) 
				{
					// Fall back on a dummy data source if a sample data table cannot be created.
					if (dummyDataTable == null) 
					{
						dummyDataTable = DesignTimeData.CreateDummyDataTable();
					}

					dataTable = dummyDataTable;
					dummyDataSource = true;
				}
			}

			IEnumerable liveDataSource = DesignTimeData.GetDesignTimeDataSource(dataTable, minimumRows);
			return liveDataSource;
		}

		public override string GetDesignTimeHtml() 
		{
			TemplatedList control = (TemplatedList)Component;
			string designTimeHTML = null;
			bool hasATemplate = this.TemplatesExist;

			if (hasATemplate) 
			{
				bool dummyDataSource;
				IEnumerable designTimeDataSource = GetDesignTimeDataSource(5, out dummyDataSource);

				try 
				{
					control.DataSource = designTimeDataSource;
					control.DataBind();

					designTimeHTML = base.GetDesignTimeHtml();
				}
				finally 
				{
					control.DataSource = null;
				}
			}
			else 
			{
				designTimeHTML = GetEmptyDesignTimeHtml();
			}

			return designTimeHTML;
		}

		protected override string GetEmptyDesignTimeHtml() 
		{
			string text;

			if (CanEnterTemplateMode) 
			{
				text = "Right click and choose a set of templates to edit their content.<br>The ItemTemplate is required.";
			}
			else 
			{
				text = "Switch to HTML view to edit the control's templates.<br>The ItemTemplate is required.";
			}
			return CreatePlaceHolderDesignTimeHtml(text);
		}

		protected override string GetErrorDesignTimeHtml(Exception e) 
		{
			Debug.Fail(e.ToString());
			return CreatePlaceHolderDesignTimeHtml("There was an error rendering the control.<br>Check to make sure all properties are valid.");
		}

		public override IEnumerable GetTemplateContainerDataSource(string templateName) 
		{
			return ((IDataSourceProvider)this).GetResolvedSelectedDataSource();
		}

		public override string GetTemplateContainerDataItemProperty(string templateName) 
		{
			return "DataItem";
		}

		public override string GetTemplateContent(ITemplateEditingFrame editingFrame, string templateName, out bool allowEditing) 
		{
			Debug.Assert(editingFrame.Verb.Index == 0);
			Debug.Assert(templateName.Equals("ItemTemplate"));
			allowEditing = true;

			ITemplate template = ((TemplatedList)Component).ItemTemplate;
			string templateContent = String.Empty;

			if (template != null) 
			{
				templateContent = GetTextFromTemplate(template);
			}

			return templateContent;
		}

		public override void OnComponentChanged(object sender, ComponentChangedEventArgs e) 
		{
			if (e.Member != null) 
			{
				string memberName = e.Member.Name;
				if (memberName.Equals("DataSource") || memberName.Equals("DataMember")) 
				{
					OnDataSourceChanged();
				}
				else if (memberName.Equals("ItemStyle")) 
				{
					OnStylesChanged();
				}
			}

			base.OnComponentChanged(sender, e);
		}

		protected virtual void OnDataSourceChanged() 
		{
			designTimeDataTable = null;
		}

		protected void OnStylesChanged() 
		{
			OnTemplateEditingVerbsChanged();
		}

		protected void OnTemplateEditingVerbsChanged() 
		{
			templateVerbsDirty = true;
		}

		protected override void PreFilterProperties(IDictionary properties) 
		{
			base.PreFilterProperties(properties);

			PropertyDescriptor prop;

			prop = (PropertyDescriptor)properties["DataSource"];
			Debug.Assert(prop != null);
			prop = TypeDescriptor.CreateProperty(this.GetType(), prop,
				new Attribute[] {
									new TypeConverterAttribute(typeof(DataSourceConverter))
								});
			properties["DataSource"] = prop;
		}

		public override void SetTemplateContent(ITemplateEditingFrame editingFrame, string templateName, string templateContent) 
		{
			Debug.Assert(editingFrame.Verb.Index == 0);
			Debug.Assert(templateName.Equals("ItemTemplate"));

			ITemplate template = null;

			if ((templateContent != null) && (templateContent.Length != 0)) 
			{
				template = GetTemplateFromText(templateContent);
			}

			((TemplatedList)Component).ItemTemplate = template;
		}


		IEnumerable IDataSourceProvider.GetResolvedSelectedDataSource() 
		{
			return (IEnumerable)((IDataSourceProvider)this).GetSelectedDataSource();
		}

		object IDataSourceProvider.GetSelectedDataSource() 
		{
			object selectedDataSource = null;
			string dataSource = null;

			DataBinding binding = DataBindings["DataSource"];
			if (binding != null) 
			{
				dataSource = binding.Expression;
			}

			if (dataSource != null) 
			{
				ISite componentSite = Component.Site;
				if (componentSite != null) 
				{
					IContainer container = (IContainer)componentSite.GetService(typeof(IContainer));

					if (container != null) 
					{
						IComponent comp = container.Components[dataSource];
						if (comp is IEnumerable) 
						{
							selectedDataSource = comp;
						}
					}
				}
			}

			return selectedDataSource;
		}
	}
}
